<?php

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2006 Elmar Hinz
 *  Contact: elmar.hinz@team-red.net
 *  All rights reserved
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***************************************************************/

require_once(t3lib_extMgm::extPath('div') . 'class.tx_div.php');
tx_div::load('tx_lib_object');

class tx_articles_models_articles{

	var $insertFields = array('title', 'teaser', 'abstract', 'article', 'category', 'type');
	var $searchFields = array('title', 'teaser', 'abstract', 'article');
	var $selectFields = array('uid', 'tstamp', 'category', 'type', 'title');
	var $languageUid = 0;
	var $articeFolderUid = 0;
	var $categoryFolderUid = 0;
	var $typeFolderUid = 0;
	var $categoryLoader; // Singleton
	var $typeLoader;     // Singleton

	function setLanguage($uid){
		$this->languageUid = $uid;
	}

	function setArticleFolder($uid){
		$this->articleFolderUid = $uid;
	}

	function setCategoryFolder($uid){
		$this->categoryFolderUid = $uid;
	}

	function setTypeFolder($uid){
		$this->typeFolderUid = $uid;
	}

	function insert($parameterObject){
		for($parameterObject->rewind(); $parameterObject->valid(); $parameterObject->next()){
			$field = $parameterObject->key();
			if(!in_array($field, $this->insertFields)){
				return FALSE;
			}
		}
		$parameterObject->set('tstamp',time());
		$parameterObject->set('hidden', 1);
		$parameterObject->set('pid', $this->articleFolderUid);
		$query = $GLOBALS['TYPO3_DB']->INSERTquery('tx_articles_articles', $parameterObject->getArrayCopy());
		$res = $GLOBALS['TYPO3_DB']->sql(TYPO3_db, $query);
		if($res) {
			return TRUE;
		}
	}

	function search($parameterObject = null, $orderObject = null, $searchWord = null, $showHidden=FALSE){
		if(!$parameterObject && !$searchWord){
			return;
		}

		$where = ' deleted = 0 ';
		$where .= ' AND pid =  ' . (int) $this->articleFolderUid;
		if(!$showHidden){
			$where .= ' AND hidden = 0 ';
		}
		// selections
		if(is_object($parameterObject)){
			foreach($this->selectFields as $field){
				if($parameterObject->get($field) != null) {
					$where .= ' AND ' . $field . ' = ' 
						. $GLOBALS['TYPO3_DB']->fullQuoteStr($parameterObject->get($field), 
								'tx_articles_articles');
				}
			}
		}

		// search
		if(isset($searchWord)){
			$or = ' 0 ';
			foreach($this->searchFields as $field){
				$or .= ' OR ' . $field . ' LIKE \'%' 
					. $GLOBALS['TYPO3_DB']->quoteStr($searchWord, 'tx_articles_articles')
					. '%\' ';
			}
			$where .= ' AND ( ' . $or . ' ) ';
		}
		$query= $GLOBALS['TYPO3_DB']->SELECTquery('*', 'tx_articles_articles', $where);
		$res = $GLOBALS['TYPO3_DB']->sql(TYPO3_db, $query);
		if($res){
			$return = $this->_exportList($this->_makeList($res));
			return $return;
		}
	}

	function getById($id){
		$where = ' deleted = 0 ';
		$where .= ' AND pid =  ' . (int) $this->articleFolderUid;
		$where .= ' AND uid=' . (int) $id;
		$query= $GLOBALS['TYPO3_DB']->SELECTquery('*', 'tx_articles_articles', $where);
		$res = $GLOBALS['TYPO3_DB']->sql(TYPO3_db, $query);
		if($res){
			return $this->_exportRow($this->_makeRow($res));
		}
	}

	function _makeList($result){
		$list = new tx_lib_object(array());
		while($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)){
			$list->append(new tx_lib_object($row));
		}
		return $list;
	}

	function _makeRow($result){
		if($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($result)){
			return new tx_lib_object($row);
		}
	}

	function _exportList($list){
		if(is_object($list)){
			for($list->rewind(); $list->valid(); $list->next()){
				$list->set($list->key(), $this->_exportRow($list->current()));
			}
			$list->rewind();
			return $list;
		}
	}

	function _exportRow($row){
		if(is_object($row)){
			$categoryLoader =& $this->categoryLoader();
			$row->set('category', $categoryLoader->get($row->get('category')));
			$typeLoader =& $this->typeLoader();
			$row->set('type', $typeLoader->get($row->get('type')));
			return $row;
		}
	}

	function categoryLoader(){
		if(!is_object($this->categoryLoader)){
			$this->categoryLoader = tx_div::makeInstance('tx_articles_models_categories');
			$this->categoryLoader->setTable('tx_articles_categories');
			$this->categoryLoader->setLanguage($this->languageUid);
			$this->categoryLoader->setFolder($this->categoryFolderUid);
			$this->categoryLoader->load();
		}
		return $this->categoryLoader;
	}

	function typeLoader(){
		if(!is_object($this->typeLoader)){
			$this->typeLoader = tx_div::makeInstance('tx_articles_models_categories');
			$this->typeLoader->setTable('tx_articles_types');
			$this->typeLoader->setLanguage($this->languageUid);
			$this->typeLoader->setFolder($this->typeFolderUid);
			$this->typeLoader->load();
		}
		return $this->typeLoader;
	}

}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/articles/models/class.tx_articles_models_articles.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/articles/models/class.tx_articles_models_articles.php']);
}
?>
