<?php

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2006 Elmar Hinz
 *  Contact: elmar.hinz@team-red.net
 *  All rights reserved
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***************************************************************/

// Set configuration first
$key = 'articles';   // extension key
$class = 'tx_articles_tests_categories_testcase'; // class name
$useDB = TRUE; // class name (Also loads the TYPO3 configuration files to fetch the passwords.)

// Fix part to set before class definition
error_reporting (E_ALL ^ E_NOTICE);
if(!defined('PATH_site')) {              // If running from command line
    tx_articles_tests_categories_testcase_init($key);
}
require_once(t3lib_extMgm::extPath('t3unit') . 'class.tx_t3unit_testcase.php');

/**
 * Test class for tx_articles_tests_categories
 */
class tx_articles_tests_categories_testcase extends tx_t3unit_testcase  {

    var $title;
    var $timestamp;
    var $pid = 999799;
    var $table = 'tx_articles_categories';
    var $ids = array(); // autoincrement ids of the inserted fields
    var $count = 3;  // count of entries to create

    /****************************************************************
     * tests
     ****************************************************************/

    function testLoad(){
        self::assertType('tx_articles_models_categories', $this->obj);
        $this->obj->setLanguage(0);
        $this->obj->load();
        // this default language
        for($x = 0; $x < $this->count; $x++){
            self::assertEquals(0 . $this->title . $x, $this->obj->get($this->ids[$x]));
        }

        $this->obj->setLanguage(1);
        $this->obj->load();
        for($x = 0; $x < $this->count; $x++){
            self::assertEquals(1 . $this->title . $x, $this->obj->get($this->ids[$x]));
        }
    }


    /****************************************************************
     * main, setUP, tearDown
     ****************************************************************/

     public function __construct ($name) {
          parent::__construct ($name);
     }

    /**
     * Sets up the fixture, for example, open a network connection.
     * This method is called before a test is executed.
     *
     * @access protected
     */
    protected function setUp() {
        $this->title = md5(rand(0, 9999));
        $this->timestamp = time();

        for($x = 0; $x < $this->count; $x++){
            // Default language
            $fields = array(
                            'pid' => $this->pid,
                            'tstamp' => $this->timestamp,
                            'title' => 0 . $this->title . $x,
                            'sys_language_uid' => 0,
                            'l18n_parent' => 0,
                            );
            $query= $GLOBALS['TYPO3_DB']->INSERTquery('tx_articles_categories', $fields);
            $res = $GLOBALS['TYPO3_DB']->sql(TYPO3_db, $query);
            $default = $this->ids[$x] = mysql_insert_id();

            // Other language
            $fields = array(
                            'pid' => $this->pid,
                            'tstamp' => $this->timestamp,
                            'title' => 1 . $this->title . $x,
                            'sys_language_uid' => 1,
                            'l18n_parent' => $default,
                            );
            $query= $GLOBALS['TYPO3_DB']->INSERTquery('tx_articles_categories', $fields);
            $res = $GLOBALS['TYPO3_DB']->sql(TYPO3_db, $query);
            $default = $this->ids[] = mysql_insert_id();
        }
        $this->load('lib');
        $this->load('div');
        $this->load('articles');
        require_once(t3lib_extMgm::extPath('div') . 'class.tx_div.php');
        require_once(t3lib_extMgm::extPath('articles') . 'models/class.tx_articles_models_categories.php');
        $this->obj = t3lib_div::makeInstance('tx_articles_models_categories');
        $this->obj->setFolder($this->pid);
        $this->obj->setTable($this->table);
    }

    /**
     * Tears down the fixture, for example, close a network connection.
     * This method is called after a test is executed.
     *
     * @access protected
     */
    protected function tearDown() {
        $query = $GLOBALS['TYPO3_DB']->DELETEquery('tx_articles_categories', 
                                                   'tstamp=' . $this->timestamp);
        $GLOBALS['TYPO3_DB']->sql(TYPO3_db, $query);
        $this->ids = array();
    }

    /**
     * Runs the test methods of this class.
     *
     * @access public
     * @static
     */
    public static function main() {
        global $class;
        require_once "PHPUnit2/TextUI/TestRunner.php";
        $suite  = new PHPUnit2_Framework_TestSuite($class);
        $result = PHPUnit2_TextUI_TestRunner::run($suite);
    }

    function load($key){
        if(is_dir(PATH_site . 'typo3conf/ext/' . $key . '/')) {
            $GLOBALS['TYPO3_LOADED_EXT']['' . $key . '']['siteRelPath']
                = 'typo3conf/ext/' . $key . '/';
        }elseif(is_dir(PATH_site . 'typo3/ext/' . $key . '/')) {
            $GLOBALS['TYPO3_LOADED_EXT']['' . $key . '']['siteRelPath'] 
                = 'typo3/ext/' . $key . '/';
        }elseif(is_file(PATH_site . 'typo3/sysext/' . $key . '/')) {
            $GLOBALS['TYPO3_LOADED_EXT']['' . $key . '']['siteRelPath']
                = 'typo3/sysext/' . $key . '/';
        }else{
            exit(chr(10) . 'Unknown installation path for ' . $key . '');
        }
    }

}

if(T3UNIT_commandline === TRUE){
    // Fix part to set after class definition
    if (!defined('PHPUnit2_MAIN_METHOD')) {
        define('PHPUnit2_MAIN_METHOD', $class . '::main');
    }
    if (PHPUnit2_MAIN_METHOD == $class . '::main') {
        eval($class . '::main();');
    }
}

/****************************************************************
 * global functions
 */ 

function tx_articles_tests_categories_testcase_init($key){
    global $useDB;
    define('T3UNIT_commandline', TRUE);  // Remember it
    // Setup environment
    $path = realpath($_SERVER['PWD'] .'/'. $_SERVER['SCRIPT_NAME']);
    if(!preg_match('|(.*)(typo3conf.*)(' . $key . '/test)|', $path, $matches))
        if(! preg_match('|(.*)(typo3/ext.*)(' . $key . '/test)|', $path, $matches))
            if(! preg_match('|(.*)(typo3/sysext.*)(' . $key . '/test)|', $path, $matches))
                exit(chr(10) . 'Unknown installation path' . chr(10). $path . chr(10));
    define('PATH_site', $matches[1]);
    define('PATH_t3lib', PATH_site . 't3lib/');
    define('PATH_typo3conf', PATH_site . 'typo3conf/');
    require_once(PATH_t3lib . 'class.t3lib_div.php');
    require_once(PATH_t3lib . 'class.t3lib_extmgm.php');
    if($useDB) {
        require_once(PATH_t3lib.'config_default.php');
        if (!defined ('TYPO3_db')) die ('The configuration file was not included.');
        require_once(PATH_t3lib.'class.t3lib_db.php');
        $GLOBALS['TYPO3_DB'] = t3lib_div::makeInstance('t3lib_DB');
        if(! $GLOBALS['TYPO3_DB']->
           sql_pconnect(TYPO3_db_host,TYPO3_db_username,TYPO3_db_password))
            die('No database connection ' . mysql_error());
        $GLOBALS['TYPO3_DB']->sql_select_db(TYPO3_db);
    }

    $GLOBALS['TYPO3_LOADED_EXT'][$key]['siteRelPath']= $matches[2] . $key . '/';

    // t3unit to loaded extensions
    $key = 't3unit';
    if(is_dir(PATH_site . 'typo3conf/ext/' . $key . '/')) {
        $GLOBALS['TYPO3_LOADED_EXT']['' . $key . '']['siteRelPath']
            = 'typo3conf/ext/' . $key . '/';
    }elseif(is_dir(PATH_site . 'typo3/ext/' . $key . '/')) {
        $GLOBALS['TYPO3_LOADED_EXT']['' . $key . '']['siteRelPath'] 
            = 'typo3/ext/' . $key . '/';
    }elseif(is_file(PATH_site . 'typo3/sysext/' . $key . '/')) {
        $GLOBALS['TYPO3_LOADED_EXT']['' . $key . '']['siteRelPath']
            = 'typo3/sysext/' . $key . '/';
    }else{
        exit(chr(10) . 'Unknown installation path for ' . $key . '');
    }
}


if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/' . $key . '/tests/class.' . $class . '.php']) {
    include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/' . $key . '/tests/class.' . $class . '.php']);
}

?>