<?php

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2006 Elmar Hinz
 *  Contact: elmar.hinz@team-red.net
 *  All rights reserved
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ***************************************************************/

// Set configuration first
$key = 'articles';   // extension key
$class = 'tx_articles_tests_model_testcase'; // class name
$useDB = TRUE; // class name

// Fix part to set before class definition
error_reporting (E_ALL ^ E_NOTICE);
if(!defined('PATH_site')) {              // If running from command line
    tx_articles_tests_model_testcase_init($key);
}
require_once(t3lib_extMgm::extPath('t3unit') . 'class.tx_t3unit_testcase.php');

/**
 * Test class for tx_articles_tests_model
 */
class tx_articles_tests_model_testcase extends tx_t3unit_testcase  {

    var $articleFolderUid  = 111111;
    var $categoryFolderUid = 222222;
    var $typeFolderUid     = 333333;
    var $category;
    var $type;
    var $title;
    var $teaser;
    var $article;
    var $abstract;
    var $timestamp;
    var $ids = array(); // autoincrement ids of the inserted fields articles
    var $catIds = array(); // autoincrement ids of the inserted fields categories
    var $typeIds = array(); // autoincrement ids of the inserted fields types
    var $count = 3;  // count of entries to create

    /****************************************************************
     * tests
     ****************************************************************/

    function test_makeRow(){
        $query= $GLOBALS['TYPO3_DB']->SELECTquery('title, teaser', 'tx_articles_articles',
                                                  'uid=' . $this->ids[1],'', 'title');
        $res = $GLOBALS['TYPO3_DB']->sql(TYPO3_db, $query);
        $row = $this->obj->_makeRow($res);
        self::assertType('tx_lib_data', $row);
        self::assertEquals($this->title . 1, $row->get('title'));
    }

    function test_makeList(){
        $query= $GLOBALS['TYPO3_DB']->SELECTquery('title, teaser', 'tx_articles_articles',  
                                                  'tstamp=' . $this->timestamp, '', 'title');
        $res = $GLOBALS['TYPO3_DB']->sql(TYPO3_db, $query);
        $list = $this->obj->_makeList($res);
        self::assertEquals($this->count, $list->count());
        self::assertType('tx_lib_data', $list);
        self::assertType('tx_lib_data', $list->get($this->count - 1));
        self::assertNotType('tx_lib_data', $list->get($this->count));
        self::assertEquals($this->title . ($this->count - 1), 
                           $list->get($this->count - 1)->get('title'));
    }

    function testGetById(){
        for($x = 0; $x < $this->count; $x++){
            $row = $this->obj->getById($this->ids[$x]);
            self::assertType('tx_lib_data', $row);
            self::assertEquals($this->title . $x, $row->get('title'));
            self::assertEquals($this->teaser . $x, $row->get('teaser'));
            self::assertEquals($this->abstract . $x, $row->get('abstract'));
            self::assertEquals($this->article . $x, $row->get('article'));
        }
    }

    function testCategoriesDefaultLanguage(){
        for($x = 0; $x < $this->count; $x++){
            $row = $this->obj->getById($this->ids[$x]);
            self::assertType('tx_lib_data', $row);
            self::assertEquals(0 . $this->category . $x, $row->get('category'));
            self::assertEquals(0 . $this->type . $x, $row->get('type'));
        }
    }

    function testCategoriesOtherLanguage(){
        $this->obj->setLanguage(1);
        for($x = 0; $x < $this->count; $x++){
            $row = $this->obj->getById($this->ids[$x]);
            self::assertType('tx_lib_data', $row);
            self::assertEquals(1 . $this->category . $x, $row->get('category'));
            self::assertEquals(1 . $this->type . $x, $row->get('type'));
        }
    }

    function testFind(){
        for($x = 0; $x < $this->count; $x++){
            $par = array('title' => $this->title . $x, 'category' => $x, 'type' => $x );
            $par = new tx_lib_data($par);
            $list = $this->obj->search($par);
            self::assertType('tx_lib_data', $list);
            for($list->rewind(); $list->valid(); $list->next()){
                $row = $list->current();
                self::assertType('tx_lib_data', $row);
                self::assertEquals($this->title . $x, $row->get('title'));
                self::assertEquals($this->teaser . $x, $row->get('teaser'));
                self::assertEquals($this->abstract . $x, $row->get('abstract'));
                self::assertEquals($this->article . $x, $row->get('article'));
            }
            // no matches for invalid combination of search pars
            $par = array('title' => $this->title . $x, 'type' => $x + 99);
            $par = new tx_lib_data($par);
            $list = $this->obj->search($par);
            self::assertEquals(0, $list->count());          
        }
    }

    function testSearch(){
        // find all
        $searchWord = substr($this->teaser, 5 ); // teilstring
        $list = $this->obj->search(null, null, $searchWord);
        self::assertEquals($this->count, $list->count());

        // find one
        $searchWord = substr($this->teaser, 5 ) . ($this->count - 1);
        $list = $this->obj->search(null, null, $searchWord);
        self::assertEquals(1, $list->count());          
    }

    public function testInsert(){
        $x = 0;
        $pars = array(
                      'title' => $this->title,
                      'teaser' => $this->teaser,
                      'abstract' => $this->abstract,
                      'article' => $this->article,
                      'category' => $x,
                      'type' => $x,
                      );
        self::assertTrue($this->obj->insert(new tx_lib_data($pars)));
        $par = new tx_lib_data(array('tstamp' => time(), 'title' => $this->title));
        $list = $this->obj->search($par, null, null, TRUE);
        self::assertEquals(1, $list->count());
        self::assertEquals($this->title, $list->current()->get('title'));
    }

    /****************************************************************
     * main, setUP, tearDown
     ****************************************************************/

    public function __construct ($name) {
        parent::__construct ($name);
    }

    /**
     * Sets up the fixture, for example, open a network connection.
     * This method is called before a test is executed.
     *
     * @access protected
     */
    protected function setUp() {
        $this->category = md5(rand(0, 9999));
        $this->type = md5(rand(0, 9999));
        $this->title = md5(rand(0, 9999));
        $this->teaser = md5(rand(0, 9999));
        $this->article = md5(rand(0, 9999));
        $this->abstract = md5(rand(0, 9999));
        $this->timestamp = time();

        for($x = 0; $x < $this->count; $x++){
            // Categories
            $fields = array(
                            'pid' => $this->categoryFolderUid,
                            'tstamp' => $this->timestamp,
                            'title' => 0 . $this->category . $x,
                            'sys_language_uid' => 0,
                            'l18n_parent' => 0,
                            );
            $query= $GLOBALS['TYPO3_DB']->INSERTquery('tx_articles_categories', $fields);
            $res = $GLOBALS['TYPO3_DB']->sql(TYPO3_db, $query);
            $default = $this->catIds[$x] = mysql_insert_id();

            $fields = array(
                            'pid' => $this->categoryFolderUid,
                            'tstamp' => $this->timestamp,
                            'title' => 1 . $this->category . $x,
                            'sys_language_uid' => 1,
                            'l18n_parent' => $default,
                            );
            $query= $GLOBALS['TYPO3_DB']->INSERTquery('tx_articles_categories', $fields);
            $res = $GLOBALS['TYPO3_DB']->sql(TYPO3_db, $query);

            // Types
            $fields = array(
                            'pid' => $this->typeFolderUid,
                            'tstamp' => $this->timestamp,
                            'title' => 0 . $this->type . $x,
                            'sys_language_uid' => 0,
                            'l18n_parent' => 0,
                            );
            $query= $GLOBALS['TYPO3_DB']->INSERTquery('tx_articles_types', $fields);
            $res = $GLOBALS['TYPO3_DB']->sql(TYPO3_db, $query);
            $default = $this->typeIds[$x] = mysql_insert_id();

            $fields = array(
                            'pid' => $this->typeFolderUid,
                            'tstamp' => $this->timestamp,
                            'title' => 1 . $this->type . $x,
                            'sys_language_uid' => 1,
                            'l18n_parent' => $default,
                            );
            $query= $GLOBALS['TYPO3_DB']->INSERTquery('tx_articles_types', $fields);
            $res = $GLOBALS['TYPO3_DB']->sql(TYPO3_db, $query);

            // Articles
            $fields = array(
                            'pid' => $this->articleFolderUid,
                            'tstamp' => $this->timestamp,
                            'title' => $this->title . $x,
                            'teaser' => $this->teaser . $x,
                            'abstract' => $this->abstract . $x,
                            'article' => $this->article . $x,
                            'category' => $this->catIds[$x],
                            'type' => $this->typeIds[$x],
                            );
            $query= $GLOBALS['TYPO3_DB']->INSERTquery('tx_articles_articles', $fields);
            $res = $GLOBALS['TYPO3_DB']->sql(TYPO3_db, $query);
            $this->ids[$x] = mysql_insert_id();            

        }
        $this->load('lib');
        $this->load('div');
        $this->load('articles');
        require_once(t3lib_extMgm::extPath('div') . 'class.tx_div.php');
        require_once(t3lib_extMgm::extPath('articles') 
                     . 'models/class.tx_articles_models_articles.php');
        $this->obj = t3lib_div::makeInstance('tx_articles_models_articles');
        $this->obj->setArticleFolder($this->articleFolderUid);
        $this->obj->setCategoryFolder($this->categoryFolderUid);
        $this->obj->setTypeFolder($this->typeFolderUid);
    }

    /**
     * Tears down the fixture, for example, close a network connection.
     * This method is called after a test is executed.
     *
     * @access protected
     */
    protected function tearDown() {
        $query = $GLOBALS['TYPO3_DB']->DELETEquery(tx_articles_articles, 
                                                   'tstamp=' . $this->timestamp);
        $GLOBALS['TYPO3_DB']->sql(TYPO3_db, $query);
        $query = $GLOBALS['TYPO3_DB']->DELETEquery(tx_articles_categories, 
                                                   'tstamp=' . $this->timestamp);
        $GLOBALS['TYPO3_DB']->sql(TYPO3_db, $query);
        $query = $GLOBALS['TYPO3_DB']->DELETEquery(tx_articles_types, 
                                                   'tstamp=' . $this->timestamp);
        $GLOBALS['TYPO3_DB']->sql(TYPO3_db, $query);
        $this->ids = array();
        $this->catIds = array();
        $this->typeIds = array();
    }

    /**
     * Load extension by key
     *
     * @access protected
     * @par    string         extension key
     * @return void    
     * 
     */
    protected function load($key){
        if(is_dir(PATH_site . 'typo3conf/ext/' . $key . '/')) {
            $GLOBALS['TYPO3_LOADED_EXT']['' . $key . '']['siteRelPath']
                = 'typo3conf/ext/' . $key . '/';
        }elseif(is_dir(PATH_site . 'typo3/ext/' . $key . '/')) {
            $GLOBALS['TYPO3_LOADED_EXT']['' . $key . '']['siteRelPath'] 
                = 'typo3/ext/' . $key . '/';
        }elseif(is_file(PATH_site . 'typo3/sysext/' . $key . '/')) {
            $GLOBALS['TYPO3_LOADED_EXT']['' . $key . '']['siteRelPath']
                = 'typo3/sysext/' . $key . '/';
        }else{
            exit(chr(10) . 'Unknown installation path for ' . $key . '');
        }
    }

    /**
     * Runs the test methods of this class.
     *
     * @access public
     * @static
     */
    public static function main() {
        global $class;
        require_once "PHPUnit2/TextUI/TestRunner.php";
        $suite  = new PHPUnit2_Framework_TestSuite($class);
        $result = PHPUnit2_TextUI_TestRunner::run($suite);
    }

}

if(T3UNIT_commandline === TRUE){
    // Fix part to set after class definition
    if (!defined('PHPUnit2_MAIN_METHOD')) {
        define('PHPUnit2_MAIN_METHOD', $class . '::main');
    }
    if (PHPUnit2_MAIN_METHOD == $class . '::main') {
        eval($class . '::main();');
    }
}

/****************************************************************
 * global function
 *
 * rename tx_***_testcase_init to unique name to prevent conflicts
 */ 

function tx_articles_tests_model_testcase_init($key){
    global $useDB;
    define('T3UNIT_commandline', TRUE);  // Remember it
    // Setup environment
    $path = realpath($_SERVER['PWD'] .'/'. $_SERVER['SCRIPT_NAME']);
    if(!preg_match('|(.*)(typo3conf.*)(' . $key . '/test)|', $path, $matches))
        if(! preg_match('|(.*)(typo3/ext.*)(' . $key . '/test)|', $path, $matches))
            if(! preg_match('|(.*)(typo3/sysext.*)(' . $key . '/test)|', $path, $matches))
                exit(chr(10) . 'Unknown installation path' . chr(10). $path . chr(10));
    define('PATH_site', $matches[1]);
    define('PATH_t3lib', PATH_site . 't3lib/');
    define('PATH_typo3conf', PATH_site . 'typo3conf/');
    require_once(PATH_t3lib . 'class.t3lib_div.php');
    require_once(PATH_t3lib . 'class.t3lib_extmgm.php');
    if($useDB) {
        require_once(PATH_t3lib.'config_default.php');
        if (!defined ('TYPO3_db')) die ('The configuration file was not included.');
        require_once(PATH_t3lib.'class.t3lib_db.php');
        $GLOBALS['TYPO3_DB'] = t3lib_div::makeInstance('t3lib_DB');
        if(! $GLOBALS['TYPO3_DB']->
           sql_pconnect(TYPO3_db_host,TYPO3_db_username,TYPO3_db_password))
            die('No database connection ' . mysql_error());
        $GLOBALS['TYPO3_DB']->sql_select_db(TYPO3_db);
    }

    $GLOBALS['TYPO3_LOADED_EXT'][$key]['siteRelPath']= $matches[2] . $key . '/';

    // t3unit to loaded extensions
    $key = 't3unit';
    if(is_dir(PATH_site . 'typo3conf/ext/' . $key . '/')) {
        $GLOBALS['TYPO3_LOADED_EXT']['' . $key . '']['siteRelPath']
            = 'typo3conf/ext/' . $key . '/';
    }elseif(is_dir(PATH_site . 'typo3/ext/' . $key . '/')) {
        $GLOBALS['TYPO3_LOADED_EXT']['' . $key . '']['siteRelPath'] 
            = 'typo3/ext/' . $key . '/';
    }elseif(is_file(PATH_site . 'typo3/sysext/' . $key . '/')) {
        $GLOBALS['TYPO3_LOADED_EXT']['' . $key . '']['siteRelPath']
            = 'typo3/sysext/' . $key . '/';
    }else{
        exit(chr(10) . 'Unknown installation path for ' . $key . '');
    }
}


if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/' . $key . '/tests/class.' . $class . '.php']) {
    include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/' . $key . '/tests/class.' . $class . '.php']);
}

?>
